{% extends "django_mailman3/profile/base.html" %}

{% load i18n %}
{% load bootstrap_tags %}

{% block user_profile_content %}

    <form method="POST" action="{% url 'account_change_password' %}" class="password_change form-horizontal">
        {% bootstrap_form_horizontal form 2 4 %}
        <div class="form-group">
          <div class="offset-md-2 col-md-10">
            <button class="btn btn-primary" type="submit" name="action">{% trans "Change Password" %}</button>
          </div>
        </div>
    </form>

{% endblock %}
